/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s):
 */

package activity;

import java.util.Collections;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.InitializationException;
import org.objectweb.dream.InterruptedPullException;
import org.objectweb.dream.Pull;
import org.objectweb.dream.PullException;
import org.objectweb.dream.control.activity.Util;
import org.objectweb.dream.control.activity.task.AbstractTask;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 *  
 */
public class ClientImpl extends AbstractComponent
{

  protected Pull           inPullItf;
  protected MessageManager messageManagerItf;

  protected void initComponent(Component componentItf)
      throws InitializationException
  {
    super.initComponent(componentItf);
    try
    {
      Util.addTask(componentItf, new PullTask(), Collections.EMPTY_MAP);
      logger.log(BasicLevel.INFO, "task added");
    }
    catch (Exception e)
    {
      throw new InitializationException(e, componentItf, "Can't add task");
    }
  }

  protected class PullTask extends AbstractTask
  {

    PullTask()
    {
      super("PullTask");
    }

    /**
     * @see org.objectweb.dream.control.activity.task.Task#execute(Object)
     */
    public Object execute(Object hints) throws InterruptedException
    {
      Message message = null;
      try
      {
        message = inPullItf.pull(null);
      }
      catch (PullException e)
      {
        System.out.println("An exeception occurs while pulling a message : "
            + e);
        if (e instanceof InterruptedPullException)
        {
          System.out.println("The pull method has been interrupted");
        }
        return STOP_EXECUTING;
      }
      System.out.println("PullComponent has received a message : " + message);
      messageManagerItf.deleteMessage(message);
      return EXECUTE_AGAIN;
    }
  } // -------------------------------------------------------------------------

  // Implementation of the BindingController interface
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Pull.IN_PULL_ITF_NAME))
    {
      inPullItf = (Pull) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Pull.IN_PULL_ITF_NAME, MessageManager.ITF_NAME};
  }

}