/**
 * DREAM
 * Copyright (C) 2003 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: matthieu.leclercq@inrialpes.fr
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.queue;

import org.objectweb.fractal.api.control.AttributeController;

/**
 * Key based Push/Pull queue attribute controller. It defines a policy applied
 * when the requested message cannot be found. Two default policies are provided
 * (specific implementations can define other).
 */
public interface PushPullKeyQueueAttributeController
    extends
      AttributeController
{

  /**
   * "Message not found" policy that returns <code>null</code> as result of
   * the pull method.
   */
  String RETURN_NULL_MESSAGE_NOT_FOUND_POLICY = "return-null";

  /**
   * "Message not found" policy that throw a {@link KeyNotFoundException }
   * exception.
   */
  String EXCEPTION_MESSAGE_NOT_FOUND_POLICY   = "exception";

  /**
   * Returns the policy applied when a message cannot be found.
   * 
   * @return the policy applied when a message cannot be found.
   */
  String getMessageNotFoundPolicy();

  /**
   * Sets the policy applied when a message cannot be found.
   * 
   * @param policy the policy applied when a message cannot be found.
   */
  void setMessageNotFoundPolicy(String policy);

  /**
   * Returns the name to which the key is mapped in the Pull context.
   * 
   * @return the name to which the key is mapped in the Pull context.
   */
  String getKeyName();

  /**
   * Sets the name to which the key is mapped in the Pull context.
   * 
   * @param name the name to which the key is mapped in the Pull context.
   */
  void setKeyName(String name);

}